//	yf Net list writer base
//		2005.10.16 - 2005.10.16 Yoshaiki.Ueda
#include "wx/wx.h"
#include "yfNetListWriterBase.h"

//	Net list writer	//=========================================================
yfNetListWriter::yfNetListWriter()
	: m_out(NULL)
{
}

yfNetListWriter::~yfNetListWriter()
{
	delete m_out;
}

bool yfNetListWriter::Write(const wxString& fileName, yfNetList& netList)
{
	wxFileOutputStream output(fileName);
	m_out = new wxTextOutputStream(output);
	if(!m_out) return false;
	BeginOfList();
	yfNetList::List::iterator net;
	for( net=netList.m_list.begin(); net!=netList.m_list.end(); net++ ){
		BeginOfNet(net->m_name);
		yfNetList::Net::NodeList::const_iterator node;
		for(node=net->m_nodeList.begin(); node!=net->m_nodeList.end(); node++){
			OnNode(node->m_name);
		}
		EndOfNet();
	}
	EndOfList();
	delete m_out;
	m_out = NULL;
	return true;
}

//	BSch Writer	//=============================================================
IMPLEMENT_DYNAMIC_CLASS(yfNetBschWriter, yfNetListWriter)
const unsigned yfNetBschWriter::NODE_OF_LINE = 5;

void yfNetBschWriter::BeginOfList()
{
	(*m_out) << wxT("$NETS") << endl;
}

void yfNetBschWriter::BeginOfNet(const wxString& name)
{
	(*m_out) << name << wxT(";\t");
	m_nodeCount = 0;
}

void yfNetBschWriter::OnNode(const wxString& name)
{
	if(m_nodeCount>=NODE_OF_LINE){
		m_nodeCount = 0;
		(*m_out) << wxT(" ,\n\t");
	}
	if(m_nodeCount!=0){
		(*m_out) << wxT(" ");
	}
	(*m_out) << name;
	m_nodeCount++;
}

void yfNetBschWriter::EndOfNet()
{
	(*m_out) << endl;
}

void yfNetBschWriter::EndOfList()
{
	(*m_out) << wxT("$END") << endl;
}

//	PADS Writer	//=============================================================
IMPLEMENT_DYNAMIC_CLASS(yfNetPadsWriter, yfNetListWriter)
const unsigned yfNetPadsWriter::NODE_OF_LINE = 5;

void yfNetPadsWriter::BeginOfList()
{
	(*m_out) << wxT("*PADS*") << endl << endl;
	(*m_out) << wxT("*NET*") << endl << endl;
}

void yfNetPadsWriter::BeginOfNet(const wxString& name)
{
	(*m_out) << wxT("*SIG ") << name << endl;
	m_nodeCount = 0;
}

void yfNetPadsWriter::OnNode(const wxString& name)
{
	if(m_nodeCount>=NODE_OF_LINE){
		m_nodeCount = 0;
		(*m_out) << endl;
	}
	if(m_nodeCount!=0){
		(*m_out) << wxT('\t');
	}
	(*m_out) << name;
	m_nodeCount++;
}

void yfNetPadsWriter::EndOfNet()
{
	(*m_out) << endl << endl;
}

void yfNetPadsWriter::EndOfList()
{
	(*m_out) << wxT("*END*") << endl;
}

//	Zuken ECF Writer	//=====================================================
IMPLEMENT_DYNAMIC_CLASS(yfNetECFWriter, yfNetListWriter)
const unsigned yfNetECFWriter::NODE_OF_LINE = 7;

void yfNetECFWriter::BeginOfList()
{
	(*m_out) << wxT("$CCF {") << endl;
	(*m_out) << wxT(" DEFINITION {") << endl;
	(*m_out) << wxT(" }") << endl;
	(*m_out) << wxT(" NET {") << endl;
}

void yfNetECFWriter::BeginOfNet(const wxString& name)
{
	(*m_out) << wxT("\t") << name << wxT(": ");
	m_nodeCount = 0;
}

void yfNetECFWriter::OnNode(const wxString& name)
{
	if(m_nodeCount>=NODE_OF_LINE){
		m_nodeCount = 0;
		(*m_out) << wxT(",\n\t\t");
	}
	if(m_nodeCount!=0){
		(*m_out) << wxT(", ");
	}
	wxString node = name;
	node.Replace(wxT("."),wxT("("));
	node.Append(wxT(')'));
	(*m_out) << node;
	m_nodeCount++;
}

void yfNetECFWriter::EndOfNet()
{
	(*m_out) << wxT(";") << endl;
}

void yfNetECFWriter::EndOfList()
{
	(*m_out) << wxT(" }") << endl;
	(*m_out) << wxT("}") << endl;
}



//	Protel Writer	//=============================================================
IMPLEMENT_DYNAMIC_CLASS(yfNetProtelWriter, yfNetListWriter)

void yfNetProtelWriter::BeginOfList()
{
}

void yfNetProtelWriter::BeginOfNet(const wxString& name)
{
	(*m_out) << wxT("(") << endl;
	(*m_out) << name <<  endl;
}

void yfNetProtelWriter::OnNode(const wxString& name)
{
	wxString node = name;
	node.Replace(wxT("."),wxT("-"));
	(*m_out) << node  << endl;
}

void yfNetProtelWriter::EndOfNet()
{
	(*m_out)<< wxT(")") << endl;
}

void yfNetProtelWriter::EndOfList()
{
	
}
